@@ -2,6 +2,7 @@ mod args;
 mod debug_query;
 mod error_context;
 mod inspect;
+mod print_diff;
 mod rule_overwrite;
 mod worker;
 
@@ -9,12 +10,13 @@ pub use args::{ContextArgs, InputArgs, OutputArgs, OverwriteArgs};
 pub use debug_query::DebugFormat;
 pub use error_context::{exit_with_error, ErrorContext};
 pub use inspect::{FileTrace, Granularity, RuleTrace, RunTrace, ScanTrace};
+pub use print_diff::DiffStyles;
 pub use rule_overwrite::RuleOverwrite;
 pub use worker::{Items, PathWorker, StdInWorker, Worker};
 
 use crate::lang::SgLang;
 
-use anyhow::{Context, Result};
+use anyhow::{anyhow, Context, Result};
 use crossterm::{
   cursor::MoveTo,
   event::{self, Event, KeyCode},
@@ -23,7 +25,7 @@ use crossterm::{
   terminal::{Clear, ClearType},
 };
 
-use ast_grep_config::{CombinedScan, PreScan, RuleCollection};
+use ast_grep_config::RuleCollection;
 use ast_grep_core::Pattern;
 use ast_grep_core::{Matcher, StrDoc};
 use ast_grep_language::Language;
@@ -90,103 +92,92 @@ pub fn prompt(prompt_text: &str, letters: &str, default: Option<char>) -> Result
   }
 }
 
-fn read_file(path: &Path) -> Option<String> {
-  let file_content = read_to_string(path)
-    .with_context(|| format!("Cannot read file {}", path.to_string_lossy()))
-    .map_err(|err| eprintln!("{err:#}"))
-    .ok()?;
+fn read_file(path: &Path) -> Result<String> {
+  let file_content =
+    read_to_string(path).with_context(|| format!("Cannot read file {}", path.to_string_lossy()))?;
   // skip large files or empty file
-  if file_too_large(&file_content) || file_content.is_empty() {
-    // TODO add output
-    None
+  if file_too_large(&file_content) {
+    Err(anyhow!("File is too large"))
+  } else if file_content.is_empty() {
+    Err(anyhow!("File is empty"))
   } else {
-    Some(file_content)
+    Ok(file_content)
   }
 }
 
-fn filter(
-  grep: &AstGrep,
+fn collect_file_stats(
   path: &Path,
   lang: SgLang,
   configs: &RuleCollection<SgLang>,
   rule_stats: &ScanTrace,
-) -> Option<PreScan> {
+) -> Result<()> {
   let rules = configs.get_rule_from_lang(path, lang);
-  rule_stats.print_file(path, lang, &rules).ok()?;
-  let combined = CombinedScan::new(rules);
-  let pre_scan = combined.find(grep);
-  if pre_scan.is_empty() {
-    None
-  } else {
-    Some(pre_scan)
-  }
+  rule_stats.print_file(path, lang, &rules)?;
+  Ok(())
 }
 
 pub fn filter_file_interactive(
   path: &Path,
   configs: &RuleCollection<SgLang>,
   trace: &ScanTrace,
-) -> Option<Vec<(PathBuf, AstGrep, PreScan)>> {
-  let lang = SgLang::from_path(path)?;
+) -> Result<Vec<(PathBuf, AstGrep)>> {
+  let Some(lang) = SgLang::from_path(path) else {
+    return Ok(vec![]);
+  };
   let file_content = read_file(path)?;
   let grep = lang.ast_grep(file_content);
-  let mut ret = vec![];
-  let root = filter(&grep, path, lang, configs, trace)
-    .map(|pre_scan| (path.to_path_buf(), grep.clone(), pre_scan));
-  ret.extend(root);
+  collect_file_stats(path, lang, configs, trace)?;
+  let mut ret = vec![(path.to_path_buf(), grep.clone())];
   if let Some(injected) = lang.injectable_sg_langs() {
     let docs = grep.inner.get_injections(|s| SgLang::from_str(s).ok());
     let inj = injected.filter_map(|l| {
       let doc = docs.iter().find(|d| *d.lang() == l)?;
       let grep = AstGrep { inner: doc.clone() };
-      let pre_scan = filter(&grep, path, l, configs, trace)?;
-      Some((path.to_path_buf(), grep, pre_scan))
+      collect_file_stats(path, l, configs, trace).ok()?;
+      Some((path.to_path_buf(), grep))
     });
     ret.extend(inj)
   }
-  Some(ret)
+  Ok(ret)
 }
 
+// sub_matchers are the injected languages
+// e.g. js/css in html
 pub fn filter_file_pattern(
   path: &Path,
   lang: SgLang,
   root_matcher: Option<Pattern<SgLang>>,
-  matchers: impl Iterator<Item = (SgLang, Pattern<SgLang>)>,
-) -> Option<Vec<(MatchUnit<Pattern<SgLang>>, SgLang)>> {
+  sub_matchers: impl Iterator<Item = (SgLang, Pattern<SgLang>)>,
+) -> Result<Vec<MatchUnit<Pattern<SgLang>>>> {
   let file_content = read_file(path)?;
   let grep = lang.ast_grep(&file_content);
-  let do_match = |ast_grep: AstGrep, matcher: Pattern<SgLang>, lang: SgLang| {
+  let do_match = |ast_grep: AstGrep, matcher: Pattern<SgLang>| {
     let fixed = matcher.fixed_string();
     if !fixed.is_empty() && !file_content.contains(&*fixed) {
       return None;
     }
     let has_match = ast_grep.root().find(&matcher).is_some();
-    has_match.then(|| {
-      (
-        MatchUnit {
-          grep: ast_grep,
-          path: path.to_path_buf(),
-          matcher,
-        },
-        lang,
-      )
+    has_match.then(|| MatchUnit {
+      grep: ast_grep,
+      path: path.to_path_buf(),
+      matcher,
     })
   };
   let mut ret = vec![];
   if let Some(matcher) = root_matcher {
-    ret.extend(do_match(grep.clone(), matcher, lang));
+    ret.extend(do_match(grep.clone(), matcher));
   }
   let injections = grep.inner.get_injections(|s| SgLang::from_str(s).ok());
-  for (i_lang, matcher) in matchers {
+  for (i_lang, matcher) in sub_matchers {
     let Some(injection) = injections.iter().find(|i| *i.lang() == i_lang) else {
       continue;
     };
     let injected = AstGrep {
       inner: injection.clone(),
     };
-    ret.extend(do_match(injected, matcher, i_lang));
+    ret.extend(do_match(injected, matcher));
   }
-  Some(ret)
+  Ok(ret)
 }
 
 const MAX_FILE_SIZE: usize = 3_000_000;
