@@ -1,5 +1,6 @@
 mod deserialize_env;
 mod nth_child;
+mod range;
 pub mod referent_rule;
 mod relational_rule;
 mod stop_by;
@@ -10,6 +11,7 @@ pub use stop_by::StopBy;
 
 use crate::maybe::Maybe;
 use nth_child::{NthChild, NthChildError, SerializableNthChild};
+use range::{RangeMatcher, RangeMatcherError, SerializableRange};
 use referent_rule::{ReferentRule, ReferentRuleError};
 use relational_rule::{Follows, Has, Inside, Precedes};
 
@@ -54,6 +56,10 @@ pub struct SerializableRule {
   /// It specifies the position in nodes' sibling list.
   #[serde(default, skip_serializing_if = "Maybe::is_absent", rename = "nthChild")]
   pub nth_child: Maybe<SerializableNthChild>,
+  /// `range` accepts a range object.
+  /// the target node must exactly appear in the range.
+  #[serde(default, skip_serializing_if = "Maybe::is_absent")]
+  pub range: Maybe<SerializableRange>,
 
   // relational
   /// `inside` accepts a relational rule object.
@@ -103,6 +109,7 @@ impl SerializableRule {
         kind: self.kind.into(),
         regex: self.regex.into(),
         nth_child: self.nth_child.into(),
+        range: self.range.into(),
       },
       relational: RelationalRule {
         inside: self.inside.into(),
@@ -125,6 +132,7 @@ pub struct AtomicRule {
   pub kind: Option<String>,
   pub regex: Option<String>,
   pub nth_child: Option<SerializableNthChild>,
+  pub range: Option<SerializableRange>,
 }
 #[derive(Serialize, Deserialize, Clone, JsonSchema)]
 #[serde(rename_all = "camelCase")]
@@ -205,6 +213,7 @@ pub enum Rule<L: Language> {
   Kind(KindMatcher<L>),
   Regex(RegexMatcher<L>),
   NthChild(NthChild<L>),
+  Range(RangeMatcher<L>),
   // relational
   Inside(Box<Inside<L>>),
   Has(Box<Has<L>>),
@@ -217,20 +226,6 @@ pub enum Rule<L: Language> {
   Matches(ReferentRule<L>),
 }
 impl<L: Language> Rule<L> {
-  pub fn is_atomic(&self) -> bool {
-    use Rule::*;
-    matches!(self, Pattern(_) | Kind(_) | Regex(_) | NthChild(_))
-  }
-  pub fn is_relational(&self) -> bool {
-    use Rule::*;
-    matches!(self, Inside(_) | Has(_) | Precedes(_) | Follows(_))
-  }
-
-  pub fn is_composite(&self) -> bool {
-    use Rule::*;
-    matches!(self, All(_) | Any(_) | Not(_) | Matches(_))
-  }
-
   /// Check if it has a cyclic referent rule with the id.
   pub(crate) fn check_cyclic(&self, id: &str) -> bool {
     match self {
@@ -238,10 +233,7 @@ impl<L: Language> Rule<L> {
       Rule::Any(any) => any.inner().iter().any(|r| r.check_cyclic(id)),
       Rule::Not(not) => not.inner().check_cyclic(id),
       Rule::Matches(m) => m.rule_id == id,
-      rule => {
-        debug_assert!(!rule.is_composite());
-        false
-      }
+      _ => false,
     }
   }
 
@@ -251,6 +243,7 @@ impl<L: Language> Rule<L> {
       Rule::Kind(_) => HashSet::new(),
       Rule::Regex(_) => HashSet::new(),
       Rule::NthChild(n) => n.defined_vars(),
+      Rule::Range(_) => HashSet::new(),
       Rule::Has(c) => c.defined_vars(),
       Rule::Inside(p) => p.defined_vars(),
       Rule::Precedes(f) => f.defined_vars(),
@@ -270,6 +263,7 @@ impl<L: Language> Rule<L> {
       Rule::Kind(_) => Ok(()),
       Rule::Regex(_) => Ok(()),
       Rule::NthChild(n) => n.verify_util(),
+      Rule::Range(_) => Ok(()),
       Rule::Has(c) => c.verify_util(),
       Rule::Inside(p) => p.verify_util(),
       Rule::Precedes(f) => f.verify_util(),
@@ -295,6 +289,7 @@ impl<L: Language> Matcher<L> for Rule<L> {
       Kind(kind) => kind.match_node_with_env(node, env),
       Regex(regex) => regex.match_node_with_env(node, env),
       NthChild(nth_child) => nth_child.match_node_with_env(node, env),
+      Range(range) => range.match_node_with_env(node, env),
       // relational
       Inside(parent) => match_and_add_label(&**parent, node, env),
       Has(child) => match_and_add_label(&**child, node, env),
@@ -316,6 +311,7 @@ impl<L: Language> Matcher<L> for Rule<L> {
       Kind(kind) => kind.potential_kinds(),
       Regex(regex) => regex.potential_kinds(),
       NthChild(nth_child) => nth_child.potential_kinds(),
+      Range(range) => range.potential_kinds(),
       // relational
       Inside(parent) => parent.potential_kinds(),
       Has(child) => child.potential_kinds(),
@@ -362,6 +358,8 @@ pub enum RuleSerializeError {
   WrongRegex(#[from] RegexMatcherError),
   #[error("Rule contains invalid matches reference.")]
   MatchesReference(#[from] ReferentRuleError),
+  #[error("Rule contains invalid range matcher.")]
+  InvalidRange(#[from] RangeMatcherError),
   #[error("field is only supported in has/inside.")]
   FieldNotSupported,
   #[error("Relational rule contains invalid field {0}.")]
@@ -480,6 +478,9 @@ fn deserialze_atomic_rule<L: Language>(
   if let Some(nth_child) = atomic.nth_child {
     rules.push(R::NthChild(NthChild::try_new(nth_child, env)?));
   }
+  if let Some(range) = atomic.range {
+    rules.push(R::Range(RangeMatcher::try_new(range.start, range.end)?));
+  }
   Ok(())
 }
 
@@ -591,7 +592,6 @@ kind: class_declaration
     let rule: SerializableRule = from_str(src).expect("cannot parse rule");
     let env = DeserializeEnv::new(TypeScript::Tsx);
     let rule = deserialize_rule(rule, &env).expect("should deserialize");
-    assert!(rule.is_composite());
     let root = TypeScript::Tsx.ast_grep("class A {}");
     assert!(root.root().find(rule).is_some());
   }
@@ -606,14 +606,7 @@ inside:
     let rule: SerializableRule = from_str(src).expect("cannot parse rule");
     let env = DeserializeEnv::new(TypeScript::Tsx);
     let rule = deserialize_rule(rule, &env).expect("should deserialize");
-    assert!(rule.is_composite());
-    let Rule::All(rules) = rule else {
-      panic!("should parsed as all");
-    };
-    // relational_rule should be the last
-    let rules = rules.inner();
-    assert!(rules[0].is_atomic());
-    assert!(rules[1].is_relational());
+    assert!(matches!(rule, Rule::All(_)));
   }
 
   #[test]
