@@ -1,13 +1,15 @@
 #![cfg(not(test))]
 #![cfg(feature = "python")]
+mod py_lang;
 mod py_node;
 mod range;
 mod unicode_position;
+use py_lang::register_dynamic_language;
 use py_node::{Edit, SgNode};
 use range::{Pos, Range};
 
 use ast_grep_core::{AstGrep, Language, NodeMatch, StrDoc};
-use ast_grep_language::SupportLang;
+use py_lang::PyLang;
 use pyo3::prelude::*;
 
 use unicode_position::UnicodePosition;
@@ -20,12 +22,13 @@ fn ast_grep_py(_py: Python, m: &Bound<PyModule>) -> PyResult<()> {
   m.add_class::<Range>()?;
   m.add_class::<Pos>()?;
   m.add_class::<Edit>()?;
+  m.add_function(wrap_pyfunction!(register_dynamic_language, m)?)?;
   Ok(())
 }
 
 #[pyclass]
 struct SgRoot {
-  inner: AstGrep<StrDoc<SupportLang>>,
+  inner: AstGrep<StrDoc<PyLang>>,
   filename: String,
   pub(crate) position: UnicodePosition,
 }
@@ -35,7 +38,7 @@ impl SgRoot {
   #[new]
   fn new(src: &str, lang: &str) -> Self {
     let position = UnicodePosition::new(src);
-    let lang: SupportLang = lang.parse().unwrap();
+    let lang: PyLang = lang.parse().unwrap();
     let inner = lang.ast_grep(src);
     Self {
       inner,
